# Make sure all code is loaded
import AppDrawing  # noqa: F401
import BitmapContext  # noqa: F401
import ColorAndGState  # noqa: F401
import CoordinateSystem  # noqa: F401
import DataProvidersAndConsumers  # noqa: F401
import DrawingBasics  # noqa: F401
import EPSPrinting  # noqa: F401
import FrameworkTextDrawing  # noqa: F401
import FrameworkUtilities  # noqa: F401
import ImageMasking  # noqa: F401
import Images  # noqa: F401
import MyAppController  # noqa: F401
import MyView  # noqa: F401
import PathDrawing  # noqa: F401
import PatternDrawing  # noqa: F401
import PDFHandling  # noqa: F401
import QuartzTextDrawing  # noqa: F401
import Shadings  # noqa: F401
import ShadowsAndTransparencyLayers  # noqa: F401
import Utilities  # noqa: F401
from PyObjCTools import AppHelper

AppHelper.runEventLoop()
