# This file is generated by objective.metadata
#
# Last update: Sun Oct 20 21:44:44 2019
#
# flake8: noqa

import sys

import objc

if sys.maxsize > 2 ** 32:

    def sel32or64(a, b):
        return b


else:

    def sel32or64(a, b):
        return a


misc = {}
constants = """$kCFErrorDomainCGImageMetadata@^{__CFString=}$kCGImageAnimationDelayTime@^{__CFString=}$kCGImageAnimationLoopCount@^{__CFString=}$kCGImageAnimationStartIndex@^{__CFString=}$kCGImageAuxiliaryDataInfoData@^{__CFString=}$kCGImageAuxiliaryDataInfoDataDescription@^{__CFString=}$kCGImageAuxiliaryDataInfoMetadata@^{__CFString=}$kCGImageAuxiliaryDataTypeDepth@^{__CFString=}$kCGImageAuxiliaryDataTypeDisparity@^{__CFString=}$kCGImageAuxiliaryDataTypePortraitEffectsMatte@^{__CFString=}$kCGImageAuxiliaryDataTypeSemanticSegmentationHairMatte@^{__CFString=}$kCGImageAuxiliaryDataTypeSemanticSegmentationSkinMatte@^{__CFString=}$kCGImageAuxiliaryDataTypeSemanticSegmentationTeethMatte@^{__CFString=}$kCGImageDestinationBackgroundColor@^{__CFString=}$kCGImageDestinationDateTime@^{__CFString=}$kCGImageDestinationEmbedThumbnail@^{__CFString=}$kCGImageDestinationImageMaxPixelSize@^{__CFString=}$kCGImageDestinationLossyCompressionQuality@^{__CFString=}$kCGImageDestinationMergeMetadata@^{__CFString=}$kCGImageDestinationMetadata@^{__CFString=}$kCGImageDestinationOptimizeColorForSharing@^{__CFString=}$kCGImageDestinationOrientation@^{__CFString=}$kCGImageMetadataEnumerateRecursively@^{__CFString=}$kCGImageMetadataNamespaceDublinCore@^{__CFString=}$kCGImageMetadataNamespaceExif@^{__CFString=}$kCGImageMetadataNamespaceExifAux@^{__CFString=}$kCGImageMetadataNamespaceExifEX@^{__CFString=}$kCGImageMetadataNamespaceIPTCCore@^{__CFString=}$kCGImageMetadataNamespaceIPTCExtension@^{__CFString=}$kCGImageMetadataNamespacePhotoshop@^{__CFString=}$kCGImageMetadataNamespaceTIFF@^{__CFString=}$kCGImageMetadataNamespaceXMPBasic@^{__CFString=}$kCGImageMetadataNamespaceXMPRights@^{__CFString=}$kCGImageMetadataPrefixDublinCore@^{__CFString=}$kCGImageMetadataPrefixExif@^{__CFString=}$kCGImageMetadataPrefixExifAux@^{__CFString=}$kCGImageMetadataPrefixExifEX@^{__CFString=}$kCGImageMetadataPrefixIPTCCore@^{__CFString=}$kCGImageMetadataPrefixIPTCExtension@^{__CFString=}$kCGImageMetadataPrefixPhotoshop@^{__CFString=}$kCGImageMetadataPrefixTIFF@^{__CFString=}$kCGImageMetadataPrefixXMPBasic@^{__CFString=}$kCGImageMetadataPrefixXMPRights@^{__CFString=}$kCGImageMetadataShouldExcludeGPS@^{__CFString=}$kCGImageMetadataShouldExcludeXMP@^{__CFString=}$kCGImageProperty8BIMDictionary@^{__CFString=}$kCGImageProperty8BIMLayerNames@^{__CFString=}$kCGImageProperty8BIMVersion@^{__CFString=}$kCGImagePropertyAPNGCanvasPixelHeight@^{__CFString=}$kCGImagePropertyAPNGCanvasPixelWidth@^{__CFString=}$kCGImagePropertyAPNGDelayTime@^{__CFString=}$kCGImagePropertyAPNGFrameInfoArray@^{__CFString=}$kCGImagePropertyAPNGLoopCount@^{__CFString=}$kCGImagePropertyAPNGUnclampedDelayTime@^{__CFString=}$kCGImagePropertyAuxiliaryData@^{__CFString=}$kCGImagePropertyAuxiliaryDataType@^{__CFString=}$kCGImagePropertyBytesPerRow@^{__CFString=}$kCGImagePropertyCIFFCameraSerialNumber@^{__CFString=}$kCGImagePropertyCIFFContinuousDrive@^{__CFString=}$kCGImagePropertyCIFFDescription@^{__CFString=}$kCGImagePropertyCIFFDictionary@^{__CFString=}$kCGImagePropertyCIFFFirmware@^{__CFString=}$kCGImagePropertyCIFFFlashExposureComp@^{__CFString=}$kCGImagePropertyCIFFFocusMode@^{__CFString=}$kCGImagePropertyCIFFImageFileName@^{__CFString=}$kCGImagePropertyCIFFImageName@^{__CFString=}$kCGImagePropertyCIFFImageSerialNumber@^{__CFString=}$kCGImagePropertyCIFFLensMaxMM@^{__CFString=}$kCGImagePropertyCIFFLensMinMM@^{__CFString=}$kCGImagePropertyCIFFLensModel@^{__CFString=}$kCGImagePropertyCIFFMeasuredEV@^{__CFString=}$kCGImagePropertyCIFFMeteringMode@^{__CFString=}$kCGImagePropertyCIFFOwnerName@^{__CFString=}$kCGImagePropertyCIFFRecordID@^{__CFString=}$kCGImagePropertyCIFFReleaseMethod@^{__CFString=}$kCGImagePropertyCIFFReleaseTiming@^{__CFString=}$kCGImagePropertyCIFFSelfTimingTime@^{__CFString=}$kCGImagePropertyCIFFShootingMode@^{__CFString=}$kCGImagePropertyCIFFWhiteBalanceIndex@^{__CFString=}$kCGImagePropertyColorModel@^{__CFString=}$kCGImagePropertyColorModelCMYK@^{__CFString=}$kCGImagePropertyColorModelGray@^{__CFString=}$kCGImagePropertyColorModelLab@^{__CFString=}$kCGImagePropertyColorModelRGB@^{__CFString=}$kCGImagePropertyDNGActiveArea@^{__CFString=}$kCGImagePropertyDNGAnalogBalance@^{__CFString=}$kCGImagePropertyDNGAntiAliasStrength@^{__CFString=}$kCGImagePropertyDNGAsShotICCProfile@^{__CFString=}$kCGImagePropertyDNGAsShotNeutral@^{__CFString=}$kCGImagePropertyDNGAsShotPreProfileMatrix@^{__CFString=}$kCGImagePropertyDNGAsShotProfileName@^{__CFString=}$kCGImagePropertyDNGAsShotWhiteXY@^{__CFString=}$kCGImagePropertyDNGBackwardVersion@^{__CFString=}$kCGImagePropertyDNGBaselineExposure@^{__CFString=}$kCGImagePropertyDNGBaselineExposureOffset@^{__CFString=}$kCGImagePropertyDNGBaselineNoise@^{__CFString=}$kCGImagePropertyDNGBaselineSharpness@^{__CFString=}$kCGImagePropertyDNGBayerGreenSplit@^{__CFString=}$kCGImagePropertyDNGBestQualityScale@^{__CFString=}$kCGImagePropertyDNGBlackLevel@^{__CFString=}$kCGImagePropertyDNGBlackLevelDeltaH@^{__CFString=}$kCGImagePropertyDNGBlackLevelDeltaV@^{__CFString=}$kCGImagePropertyDNGBlackLevelRepeatDim@^{__CFString=}$kCGImagePropertyDNGCFALayout@^{__CFString=}$kCGImagePropertyDNGCFAPlaneColor@^{__CFString=}$kCGImagePropertyDNGCalibrationIlluminant1@^{__CFString=}$kCGImagePropertyDNGCalibrationIlluminant2@^{__CFString=}$kCGImagePropertyDNGCameraCalibration1@^{__CFString=}$kCGImagePropertyDNGCameraCalibration2@^{__CFString=}$kCGImagePropertyDNGCameraCalibrationSignature@^{__CFString=}$kCGImagePropertyDNGCameraSerialNumber@^{__CFString=}$kCGImagePropertyDNGChromaBlurRadius@^{__CFString=}$kCGImagePropertyDNGColorMatrix1@^{__CFString=}$kCGImagePropertyDNGColorMatrix2@^{__CFString=}$kCGImagePropertyDNGColorimetricReference@^{__CFString=}$kCGImagePropertyDNGCurrentICCProfile@^{__CFString=}$kCGImagePropertyDNGCurrentPreProfileMatrix@^{__CFString=}$kCGImagePropertyDNGDefaultBlackRender@^{__CFString=}$kCGImagePropertyDNGDefaultCropOrigin@^{__CFString=}$kCGImagePropertyDNGDefaultCropSize@^{__CFString=}$kCGImagePropertyDNGDefaultScale@^{__CFString=}$kCGImagePropertyDNGDefaultUserCrop@^{__CFString=}$kCGImagePropertyDNGDictionary@^{__CFString=}$kCGImagePropertyDNGExtraCameraProfiles@^{__CFString=}$kCGImagePropertyDNGFixVignetteRadial@^{__CFString=}$kCGImagePropertyDNGForwardMatrix1@^{__CFString=}$kCGImagePropertyDNGForwardMatrix2@^{__CFString=}$kCGImagePropertyDNGLensInfo@^{__CFString=}$kCGImagePropertyDNGLinearResponseLimit@^{__CFString=}$kCGImagePropertyDNGLinearizationTable@^{__CFString=}$kCGImagePropertyDNGLocalizedCameraModel@^{__CFString=}$kCGImagePropertyDNGMakerNoteSafety@^{__CFString=}$kCGImagePropertyDNGMaskedAreas@^{__CFString=}$kCGImagePropertyDNGNewRawImageDigest@^{__CFString=}$kCGImagePropertyDNGNoiseProfile@^{__CFString=}$kCGImagePropertyDNGNoiseReductionApplied@^{__CFString=}$kCGImagePropertyDNGOpcodeList1@^{__CFString=}$kCGImagePropertyDNGOpcodeList2@^{__CFString=}$kCGImagePropertyDNGOpcodeList3@^{__CFString=}$kCGImagePropertyDNGOriginalBestQualityFinalSize@^{__CFString=}$kCGImagePropertyDNGOriginalDefaultCropSize@^{__CFString=}$kCGImagePropertyDNGOriginalDefaultFinalSize@^{__CFString=}$kCGImagePropertyDNGOriginalRawFileData@^{__CFString=}$kCGImagePropertyDNGOriginalRawFileDigest@^{__CFString=}$kCGImagePropertyDNGOriginalRawFileName@^{__CFString=}$kCGImagePropertyDNGPreviewApplicationName@^{__CFString=}$kCGImagePropertyDNGPreviewApplicationVersion@^{__CFString=}$kCGImagePropertyDNGPreviewColorSpace@^{__CFString=}$kCGImagePropertyDNGPreviewDateTime@^{__CFString=}$kCGImagePropertyDNGPreviewSettingsDigest@^{__CFString=}$kCGImagePropertyDNGPreviewSettingsName@^{__CFString=}$kCGImagePropertyDNGPrivateData@^{__CFString=}$kCGImagePropertyDNGProfileCalibrationSignature@^{__CFString=}$kCGImagePropertyDNGProfileCopyright@^{__CFString=}$kCGImagePropertyDNGProfileEmbedPolicy@^{__CFString=}$kCGImagePropertyDNGProfileHueSatMapData1@^{__CFString=}$kCGImagePropertyDNGProfileHueSatMapData2@^{__CFString=}$kCGImagePropertyDNGProfileHueSatMapDims@^{__CFString=}$kCGImagePropertyDNGProfileHueSatMapEncoding@^{__CFString=}$kCGImagePropertyDNGProfileLookTableData@^{__CFString=}$kCGImagePropertyDNGProfileLookTableDims@^{__CFString=}$kCGImagePropertyDNGProfileLookTableEncoding@^{__CFString=}$kCGImagePropertyDNGProfileName@^{__CFString=}$kCGImagePropertyDNGProfileToneCurve@^{__CFString=}$kCGImagePropertyDNGRawDataUniqueID@^{__CFString=}$kCGImagePropertyDNGRawImageDigest@^{__CFString=}$kCGImagePropertyDNGRawToPreviewGain@^{__CFString=}$kCGImagePropertyDNGReductionMatrix1@^{__CFString=}$kCGImagePropertyDNGReductionMatrix2@^{__CFString=}$kCGImagePropertyDNGRowInterleaveFactor@^{__CFString=}$kCGImagePropertyDNGShadowScale@^{__CFString=}$kCGImagePropertyDNGSubTileBlockSize@^{__CFString=}$kCGImagePropertyDNGUniqueCameraModel@^{__CFString=}$kCGImagePropertyDNGVersion@^{__CFString=}$kCGImagePropertyDNGWarpFisheye@^{__CFString=}$kCGImagePropertyDNGWarpRectilinear@^{__CFString=}$kCGImagePropertyDNGWhiteLevel@^{__CFString=}$kCGImagePropertyDPIHeight@^{__CFString=}$kCGImagePropertyDPIWidth@^{__CFString=}$kCGImagePropertyDepth@^{__CFString=}$kCGImagePropertyExifApertureValue@^{__CFString=}$kCGImagePropertyExifAuxDictionary@^{__CFString=}$kCGImagePropertyExifAuxFirmware@^{__CFString=}$kCGImagePropertyExifAuxFlashCompensation@^{__CFString=}$kCGImagePropertyExifAuxImageNumber@^{__CFString=}$kCGImagePropertyExifAuxLensID@^{__CFString=}$kCGImagePropertyExifAuxLensInfo@^{__CFString=}$kCGImagePropertyExifAuxLensModel@^{__CFString=}$kCGImagePropertyExifAuxLensSerialNumber@^{__CFString=}$kCGImagePropertyExifAuxOwnerName@^{__CFString=}$kCGImagePropertyExifAuxSerialNumber@^{__CFString=}$kCGImagePropertyExifBodySerialNumber@^{__CFString=}$kCGImagePropertyExifBrightnessValue@^{__CFString=}$kCGImagePropertyExifCFAPattern@^{__CFString=}$kCGImagePropertyExifCameraOwnerName@^{__CFString=}$kCGImagePropertyExifColorSpace@^{__CFString=}$kCGImagePropertyExifComponentsConfiguration@^{__CFString=}$kCGImagePropertyExifCompositeImage$kCGImagePropertyExifCompressedBitsPerPixel@^{__CFString=}$kCGImagePropertyExifContrast@^{__CFString=}$kCGImagePropertyExifCustomRendered@^{__CFString=}$kCGImagePropertyExifDateTimeDigitized@^{__CFString=}$kCGImagePropertyExifDateTimeOriginal@^{__CFString=}$kCGImagePropertyExifDeviceSettingDescription@^{__CFString=}$kCGImagePropertyExifDictionary@^{__CFString=}$kCGImagePropertyExifDigitalZoomRatio@^{__CFString=}$kCGImagePropertyExifExposureBiasValue@^{__CFString=}$kCGImagePropertyExifExposureIndex@^{__CFString=}$kCGImagePropertyExifExposureMode@^{__CFString=}$kCGImagePropertyExifExposureProgram@^{__CFString=}$kCGImagePropertyExifExposureTime@^{__CFString=}$kCGImagePropertyExifFNumber@^{__CFString=}$kCGImagePropertyExifFileSource@^{__CFString=}$kCGImagePropertyExifFlash@^{__CFString=}$kCGImagePropertyExifFlashEnergy@^{__CFString=}$kCGImagePropertyExifFlashPixVersion@^{__CFString=}$kCGImagePropertyExifFocalLenIn35mmFilm@^{__CFString=}$kCGImagePropertyExifFocalLength@^{__CFString=}$kCGImagePropertyExifFocalPlaneResolutionUnit@^{__CFString=}$kCGImagePropertyExifFocalPlaneXResolution@^{__CFString=}$kCGImagePropertyExifFocalPlaneYResolution@^{__CFString=}$kCGImagePropertyExifGainControl@^{__CFString=}$kCGImagePropertyExifGamma@^{__CFString=}$kCGImagePropertyExifISOSpeed@^{__CFString=}$kCGImagePropertyExifISOSpeedLatitudeyyy@^{__CFString=}$kCGImagePropertyExifISOSpeedLatitudezzz@^{__CFString=}$kCGImagePropertyExifISOSpeedRatings@^{__CFString=}$kCGImagePropertyExifImageUniqueID@^{__CFString=}$kCGImagePropertyExifLensMake@^{__CFString=}$kCGImagePropertyExifLensModel@^{__CFString=}$kCGImagePropertyExifLensSerialNumber@^{__CFString=}$kCGImagePropertyExifLensSpecification@^{__CFString=}$kCGImagePropertyExifLightSource@^{__CFString=}$kCGImagePropertyExifMakerNote@^{__CFString=}$kCGImagePropertyExifMaxApertureValue@^{__CFString=}$kCGImagePropertyExifMeteringMode@^{__CFString=}$kCGImagePropertyExifOECF@^{__CFString=}$kCGImagePropertyExifOffsetTime@^{__CFString=}$kCGImagePropertyExifOffsetTimeDigitized@^{__CFString=}$kCGImagePropertyExifOffsetTimeOriginal@^{__CFString=}$kCGImagePropertyExifPixelXDimension@^{__CFString=}$kCGImagePropertyExifPixelYDimension@^{__CFString=}$kCGImagePropertyExifRecommendedExposureIndex@^{__CFString=}$kCGImagePropertyExifRelatedSoundFile@^{__CFString=}$kCGImagePropertyExifSaturation@^{__CFString=}$kCGImagePropertyExifSceneCaptureType@^{__CFString=}$kCGImagePropertyExifSceneType@^{__CFString=}$kCGImagePropertyExifSensingMethod@^{__CFString=}$kCGImagePropertyExifSensitivityType@^{__CFString=}$kCGImagePropertyExifSharpness@^{__CFString=}$kCGImagePropertyExifShutterSpeedValue@^{__CFString=}$kCGImagePropertyExifSourceExposureTimesOfCompositeImage$kCGImagePropertyExifSourceImageNumberOfCompositeImage$kCGImagePropertyExifSpatialFrequencyResponse@^{__CFString=}$kCGImagePropertyExifSpectralSensitivity@^{__CFString=}$kCGImagePropertyExifStandardOutputSensitivity@^{__CFString=}$kCGImagePropertyExifSubjectArea@^{__CFString=}$kCGImagePropertyExifSubjectDistRange@^{__CFString=}$kCGImagePropertyExifSubjectDistance@^{__CFString=}$kCGImagePropertyExifSubjectLocation@^{__CFString=}$kCGImagePropertyExifSubsecTime@^{__CFString=}$kCGImagePropertyExifSubsecTimeDigitized@^{__CFString=}$kCGImagePropertyExifSubsecTimeOrginal@^{__CFString=}$kCGImagePropertyExifSubsecTimeOriginal@^{__CFString=}$kCGImagePropertyExifUserComment@^{__CFString=}$kCGImagePropertyExifVersion@^{__CFString=}$kCGImagePropertyExifWhiteBalance@^{__CFString=}$kCGImagePropertyFileContentsDictionary@^{__CFString=}$kCGImagePropertyFileSize@^{__CFString=}$kCGImagePropertyGIFCanvasPixelHeight@^{__CFString=}$kCGImagePropertyGIFCanvasPixelWidth@^{__CFString=}$kCGImagePropertyGIFDelayTime@^{__CFString=}$kCGImagePropertyGIFDictionary@^{__CFString=}$kCGImagePropertyGIFFrameInfoArray@^{__CFString=}$kCGImagePropertyGIFHasGlobalColorMap@^{__CFString=}$kCGImagePropertyGIFImageColorMap@^{__CFString=}$kCGImagePropertyGIFLoopCount@^{__CFString=}$kCGImagePropertyGIFUnclampedDelayTime@^{__CFString=}$kCGImagePropertyGPSAltitude@^{__CFString=}$kCGImagePropertyGPSAltitudeRef@^{__CFString=}$kCGImagePropertyGPSAreaInformation@^{__CFString=}$kCGImagePropertyGPSDOP@^{__CFString=}$kCGImagePropertyGPSDateStamp@^{__CFString=}$kCGImagePropertyGPSDestBearing@^{__CFString=}$kCGImagePropertyGPSDestBearingRef@^{__CFString=}$kCGImagePropertyGPSDestDistance@^{__CFString=}$kCGImagePropertyGPSDestDistanceRef@^{__CFString=}$kCGImagePropertyGPSDestLatitude@^{__CFString=}$kCGImagePropertyGPSDestLatitudeRef@^{__CFString=}$kCGImagePropertyGPSDestLongitude@^{__CFString=}$kCGImagePropertyGPSDestLongitudeRef@^{__CFString=}$kCGImagePropertyGPSDictionary@^{__CFString=}$kCGImagePropertyGPSDifferental@^{__CFString=}$kCGImagePropertyGPSHPositioningError@^{__CFString=}$kCGImagePropertyGPSImgDirection@^{__CFString=}$kCGImagePropertyGPSImgDirectionRef@^{__CFString=}$kCGImagePropertyGPSLatitude@^{__CFString=}$kCGImagePropertyGPSLatitudeRef@^{__CFString=}$kCGImagePropertyGPSLongitude@^{__CFString=}$kCGImagePropertyGPSLongitudeRef@^{__CFString=}$kCGImagePropertyGPSMapDatum@^{__CFString=}$kCGImagePropertyGPSMeasureMode@^{__CFString=}$kCGImagePropertyGPSProcessingMethod@^{__CFString=}$kCGImagePropertyGPSSatellites@^{__CFString=}$kCGImagePropertyGPSSpeed@^{__CFString=}$kCGImagePropertyGPSSpeedRef@^{__CFString=}$kCGImagePropertyGPSStatus@^{__CFString=}$kCGImagePropertyGPSTimeStamp@^{__CFString=}$kCGImagePropertyGPSTrack@^{__CFString=}$kCGImagePropertyGPSTrackRef@^{__CFString=}$kCGImagePropertyGPSVersion@^{__CFString=}$kCGImagePropertyHEICSCanvasPixelHeight@^{__CFString=}$kCGImagePropertyHEICSCanvasPixelWidth@^{__CFString=}$kCGImagePropertyHEICSDelayTime@^{__CFString=}$kCGImagePropertyHEICSDictionary@^{__CFString=}$kCGImagePropertyHEICSFrameInfoArray@^{__CFString=}$kCGImagePropertyHEICSLoopCount@^{__CFString=}$kCGImagePropertyHEICSUnclampedDelayTime@^{__CFString=}$kCGImagePropertyHasAlpha@^{__CFString=}$kCGImagePropertyHeight@^{__CFString=}$kCGImagePropertyIPTCActionAdvised@^{__CFString=}$kCGImagePropertyIPTCByline@^{__CFString=}$kCGImagePropertyIPTCBylineTitle@^{__CFString=}$kCGImagePropertyIPTCCaptionAbstract@^{__CFString=}$kCGImagePropertyIPTCCategory@^{__CFString=}$kCGImagePropertyIPTCCity@^{__CFString=}$kCGImagePropertyIPTCContact@^{__CFString=}$kCGImagePropertyIPTCContactInfoAddress@^{__CFString=}$kCGImagePropertyIPTCContactInfoCity@^{__CFString=}$kCGImagePropertyIPTCContactInfoCountry@^{__CFString=}$kCGImagePropertyIPTCContactInfoEmails@^{__CFString=}$kCGImagePropertyIPTCContactInfoPhones@^{__CFString=}$kCGImagePropertyIPTCContactInfoPostalCode@^{__CFString=}$kCGImagePropertyIPTCContactInfoStateProvince@^{__CFString=}$kCGImagePropertyIPTCContactInfoWebURLs@^{__CFString=}$kCGImagePropertyIPTCContentLocationCode@^{__CFString=}$kCGImagePropertyIPTCContentLocationName@^{__CFString=}$kCGImagePropertyIPTCCopyrightNotice@^{__CFString=}$kCGImagePropertyIPTCCountryPrimaryLocationCode@^{__CFString=}$kCGImagePropertyIPTCCountryPrimaryLocationName@^{__CFString=}$kCGImagePropertyIPTCCreatorContactInfo@^{__CFString=}$kCGImagePropertyIPTCCredit@^{__CFString=}$kCGImagePropertyIPTCDateCreated@^{__CFString=}$kCGImagePropertyIPTCDictionary@^{__CFString=}$kCGImagePropertyIPTCDigitalCreationDate@^{__CFString=}$kCGImagePropertyIPTCDigitalCreationTime@^{__CFString=}$kCGImagePropertyIPTCEditStatus@^{__CFString=}$kCGImagePropertyIPTCEditorialUpdate@^{__CFString=}$kCGImagePropertyIPTCExpirationDate@^{__CFString=}$kCGImagePropertyIPTCExpirationTime@^{__CFString=}$kCGImagePropertyIPTCExtAboutCvTerm@^{__CFString=}$kCGImagePropertyIPTCExtAboutCvTermCvId@^{__CFString=}$kCGImagePropertyIPTCExtAboutCvTermId@^{__CFString=}$kCGImagePropertyIPTCExtAboutCvTermName@^{__CFString=}$kCGImagePropertyIPTCExtAboutCvTermRefinedAbout@^{__CFString=}$kCGImagePropertyIPTCExtAddlModelInfo@^{__CFString=}$kCGImagePropertyIPTCExtArtworkCircaDateCreated@^{__CFString=}$kCGImagePropertyIPTCExtArtworkContentDescription@^{__CFString=}$kCGImagePropertyIPTCExtArtworkContributionDescription@^{__CFString=}$kCGImagePropertyIPTCExtArtworkCopyrightNotice@^{__CFString=}$kCGImagePropertyIPTCExtArtworkCopyrightOwnerID@^{__CFString=}$kCGImagePropertyIPTCExtArtworkCopyrightOwnerName@^{__CFString=}$kCGImagePropertyIPTCExtArtworkCreator@^{__CFString=}$kCGImagePropertyIPTCExtArtworkCreatorID@^{__CFString=}$kCGImagePropertyIPTCExtArtworkDateCreated@^{__CFString=}$kCGImagePropertyIPTCExtArtworkLicensorID@^{__CFString=}$kCGImagePropertyIPTCExtArtworkLicensorName@^{__CFString=}$kCGImagePropertyIPTCExtArtworkOrObject@^{__CFString=}$kCGImagePropertyIPTCExtArtworkPhysicalDescription@^{__CFString=}$kCGImagePropertyIPTCExtArtworkSource@^{__CFString=}$kCGImagePropertyIPTCExtArtworkSourceInvURL@^{__CFString=}$kCGImagePropertyIPTCExtArtworkSourceInventoryNo@^{__CFString=}$kCGImagePropertyIPTCExtArtworkStylePeriod@^{__CFString=}$kCGImagePropertyIPTCExtArtworkTitle@^{__CFString=}$kCGImagePropertyIPTCExtAudioBitrate@^{__CFString=}$kCGImagePropertyIPTCExtAudioBitrateMode@^{__CFString=}$kCGImagePropertyIPTCExtAudioChannelCount@^{__CFString=}$kCGImagePropertyIPTCExtCircaDateCreated@^{__CFString=}$kCGImagePropertyIPTCExtContainerFormat@^{__CFString=}$kCGImagePropertyIPTCExtContainerFormatIdentifier@^{__CFString=}$kCGImagePropertyIPTCExtContainerFormatName@^{__CFString=}$kCGImagePropertyIPTCExtContributor@^{__CFString=}$kCGImagePropertyIPTCExtContributorIdentifier@^{__CFString=}$kCGImagePropertyIPTCExtContributorName@^{__CFString=}$kCGImagePropertyIPTCExtContributorRole@^{__CFString=}$kCGImagePropertyIPTCExtControlledVocabularyTerm@^{__CFString=}$kCGImagePropertyIPTCExtCopyrightYear@^{__CFString=}$kCGImagePropertyIPTCExtCreator@^{__CFString=}$kCGImagePropertyIPTCExtCreatorIdentifier@^{__CFString=}$kCGImagePropertyIPTCExtCreatorName@^{__CFString=}$kCGImagePropertyIPTCExtCreatorRole@^{__CFString=}$kCGImagePropertyIPTCExtDataOnScreen@^{__CFString=}$kCGImagePropertyIPTCExtDataOnScreenRegion@^{__CFString=}$kCGImagePropertyIPTCExtDataOnScreenRegionD@^{__CFString=}$kCGImagePropertyIPTCExtDataOnScreenRegionH@^{__CFString=}$kCGImagePropertyIPTCExtDataOnScreenRegionText@^{__CFString=}$kCGImagePropertyIPTCExtDataOnScreenRegionUnit@^{__CFString=}$kCGImagePropertyIPTCExtDataOnScreenRegionW@^{__CFString=}$kCGImagePropertyIPTCExtDataOnScreenRegionX@^{__CFString=}$kCGImagePropertyIPTCExtDataOnScreenRegionY@^{__CFString=}$kCGImagePropertyIPTCExtDigitalImageGUID@^{__CFString=}$kCGImagePropertyIPTCExtDigitalSourceFileType@^{__CFString=}$kCGImagePropertyIPTCExtDigitalSourceType@^{__CFString=}$kCGImagePropertyIPTCExtDopesheet@^{__CFString=}$kCGImagePropertyIPTCExtDopesheetLink@^{__CFString=}$kCGImagePropertyIPTCExtDopesheetLinkLink@^{__CFString=}$kCGImagePropertyIPTCExtDopesheetLinkLinkQualifier@^{__CFString=}$kCGImagePropertyIPTCExtEmbdEncRightsExpr@^{__CFString=}$kCGImagePropertyIPTCExtEmbeddedEncodedRightsExpr@^{__CFString=}$kCGImagePropertyIPTCExtEmbeddedEncodedRightsExprLangID@^{__CFString=}$kCGImagePropertyIPTCExtEmbeddedEncodedRightsExprType@^{__CFString=}$kCGImagePropertyIPTCExtEpisode@^{__CFString=}$kCGImagePropertyIPTCExtEpisodeIdentifier@^{__CFString=}$kCGImagePropertyIPTCExtEpisodeName@^{__CFString=}$kCGImagePropertyIPTCExtEpisodeNumber@^{__CFString=}$kCGImagePropertyIPTCExtEvent@^{__CFString=}$kCGImagePropertyIPTCExtExternalMetadataLink@^{__CFString=}$kCGImagePropertyIPTCExtFeedIdentifier@^{__CFString=}$kCGImagePropertyIPTCExtGenre@^{__CFString=}$kCGImagePropertyIPTCExtGenreCvId@^{__CFString=}$kCGImagePropertyIPTCExtGenreCvTermId@^{__CFString=}$kCGImagePropertyIPTCExtGenreCvTermName@^{__CFString=}$kCGImagePropertyIPTCExtGenreCvTermRefinedAbout@^{__CFString=}$kCGImagePropertyIPTCExtHeadline@^{__CFString=}$kCGImagePropertyIPTCExtIPTCLastEdited@^{__CFString=}$kCGImagePropertyIPTCExtLinkedEncRightsExpr@^{__CFString=}$kCGImagePropertyIPTCExtLinkedEncodedRightsExpr@^{__CFString=}$kCGImagePropertyIPTCExtLinkedEncodedRightsExprLangID@^{__CFString=}$kCGImagePropertyIPTCExtLinkedEncodedRightsExprType@^{__CFString=}$kCGImagePropertyIPTCExtLocationCity@^{__CFString=}$kCGImagePropertyIPTCExtLocationCountryCode@^{__CFString=}$kCGImagePropertyIPTCExtLocationCountryName@^{__CFString=}$kCGImagePropertyIPTCExtLocationCreated@^{__CFString=}$kCGImagePropertyIPTCExtLocationGPSAltitude@^{__CFString=}$kCGImagePropertyIPTCExtLocationGPSLatitude@^{__CFString=}$kCGImagePropertyIPTCExtLocationGPSLongitude@^{__CFString=}$kCGImagePropertyIPTCExtLocationIdentifier@^{__CFString=}$kCGImagePropertyIPTCExtLocationLocationId@^{__CFString=}$kCGImagePropertyIPTCExtLocationLocationName@^{__CFString=}$kCGImagePropertyIPTCExtLocationProvinceState@^{__CFString=}$kCGImagePropertyIPTCExtLocationShown@^{__CFString=}$kCGImagePropertyIPTCExtLocationSublocation@^{__CFString=}$kCGImagePropertyIPTCExtLocationWorldRegion@^{__CFString=}$kCGImagePropertyIPTCExtMaxAvailHeight@^{__CFString=}$kCGImagePropertyIPTCExtMaxAvailWidth@^{__CFString=}$kCGImagePropertyIPTCExtModelAge@^{__CFString=}$kCGImagePropertyIPTCExtOrganisationInImageCode@^{__CFString=}$kCGImagePropertyIPTCExtOrganisationInImageName@^{__CFString=}$kCGImagePropertyIPTCExtPersonHeard@^{__CFString=}$kCGImagePropertyIPTCExtPersonHeardIdentifier@^{__CFString=}$kCGImagePropertyIPTCExtPersonHeardName@^{__CFString=}$kCGImagePropertyIPTCExtPersonInImage@^{__CFString=}$kCGImagePropertyIPTCExtPersonInImageCharacteristic@^{__CFString=}$kCGImagePropertyIPTCExtPersonInImageCvTermCvId@^{__CFString=}$kCGImagePropertyIPTCExtPersonInImageCvTermId@^{__CFString=}$kCGImagePropertyIPTCExtPersonInImageCvTermName@^{__CFString=}$kCGImagePropertyIPTCExtPersonInImageCvTermRefinedAbout@^{__CFString=}$kCGImagePropertyIPTCExtPersonInImageDescription@^{__CFString=}$kCGImagePropertyIPTCExtPersonInImageId@^{__CFString=}$kCGImagePropertyIPTCExtPersonInImageName@^{__CFString=}$kCGImagePropertyIPTCExtPersonInImageWDetails@^{__CFString=}$kCGImagePropertyIPTCExtProductInImage@^{__CFString=}$kCGImagePropertyIPTCExtProductInImageDescription@^{__CFString=}$kCGImagePropertyIPTCExtProductInImageGTIN@^{__CFString=}$kCGImagePropertyIPTCExtProductInImageName@^{__CFString=}$kCGImagePropertyIPTCExtPublicationEvent@^{__CFString=}$kCGImagePropertyIPTCExtPublicationEventDate@^{__CFString=}$kCGImagePropertyIPTCExtPublicationEventIdentifier@^{__CFString=}$kCGImagePropertyIPTCExtPublicationEventName@^{__CFString=}$kCGImagePropertyIPTCExtRating@^{__CFString=}$kCGImagePropertyIPTCExtRatingRatingRegion@^{__CFString=}$kCGImagePropertyIPTCExtRatingRegionCity@^{__CFString=}$kCGImagePropertyIPTCExtRatingRegionCountryCode@^{__CFString=}$kCGImagePropertyIPTCExtRatingRegionCountryName@^{__CFString=}$kCGImagePropertyIPTCExtRatingRegionGPSAltitude@^{__CFString=}$kCGImagePropertyIPTCExtRatingRegionGPSLatitude@^{__CFString=}$kCGImagePropertyIPTCExtRatingRegionGPSLongitude@^{__CFString=}$kCGImagePropertyIPTCExtRatingRegionIdentifier@^{__CFString=}$kCGImagePropertyIPTCExtRatingRegionLocationId@^{__CFString=}$kCGImagePropertyIPTCExtRatingRegionLocationName@^{__CFString=}$kCGImagePropertyIPTCExtRatingRegionProvinceState@^{__CFString=}$kCGImagePropertyIPTCExtRatingRegionSublocation@^{__CFString=}$kCGImagePropertyIPTCExtRatingRegionWorldRegion@^{__CFString=}$kCGImagePropertyIPTCExtRatingScaleMaxValue@^{__CFString=}$kCGImagePropertyIPTCExtRatingScaleMinValue@^{__CFString=}$kCGImagePropertyIPTCExtRatingSourceLink@^{__CFString=}$kCGImagePropertyIPTCExtRatingValue@^{__CFString=}$kCGImagePropertyIPTCExtRatingValueLogoLink@^{__CFString=}$kCGImagePropertyIPTCExtRegistryEntryRole@^{__CFString=}$kCGImagePropertyIPTCExtRegistryID@^{__CFString=}$kCGImagePropertyIPTCExtRegistryItemID@^{__CFString=}$kCGImagePropertyIPTCExtRegistryOrganisationID@^{__CFString=}$kCGImagePropertyIPTCExtReleaseReady@^{__CFString=}$kCGImagePropertyIPTCExtSeason@^{__CFString=}$kCGImagePropertyIPTCExtSeasonIdentifier@^{__CFString=}$kCGImagePropertyIPTCExtSeasonName@^{__CFString=}$kCGImagePropertyIPTCExtSeasonNumber@^{__CFString=}$kCGImagePropertyIPTCExtSeries@^{__CFString=}$kCGImagePropertyIPTCExtSeriesIdentifier@^{__CFString=}$kCGImagePropertyIPTCExtSeriesName@^{__CFString=}$kCGImagePropertyIPTCExtShownEvent@^{__CFString=}$kCGImagePropertyIPTCExtShownEventIdentifier@^{__CFString=}$kCGImagePropertyIPTCExtShownEventName@^{__CFString=}$kCGImagePropertyIPTCExtStorylineIdentifier@^{__CFString=}$kCGImagePropertyIPTCExtStreamReady@^{__CFString=}$kCGImagePropertyIPTCExtStylePeriod@^{__CFString=}$kCGImagePropertyIPTCExtSupplyChainSource@^{__CFString=}$kCGImagePropertyIPTCExtSupplyChainSourceIdentifier@^{__CFString=}$kCGImagePropertyIPTCExtSupplyChainSourceName@^{__CFString=}$kCGImagePropertyIPTCExtTemporalCoverage@^{__CFString=}$kCGImagePropertyIPTCExtTemporalCoverageFrom@^{__CFString=}$kCGImagePropertyIPTCExtTemporalCoverageTo@^{__CFString=}$kCGImagePropertyIPTCExtTranscript@^{__CFString=}$kCGImagePropertyIPTCExtTranscriptLink@^{__CFString=}$kCGImagePropertyIPTCExtTranscriptLinkLink@^{__CFString=}$kCGImagePropertyIPTCExtTranscriptLinkLinkQualifier@^{__CFString=}$kCGImagePropertyIPTCExtVideoBitrate@^{__CFString=}$kCGImagePropertyIPTCExtVideoBitrateMode@^{__CFString=}$kCGImagePropertyIPTCExtVideoDisplayAspectRatio@^{__CFString=}$kCGImagePropertyIPTCExtVideoEncodingProfile@^{__CFString=}$kCGImagePropertyIPTCExtVideoShotType@^{__CFString=}$kCGImagePropertyIPTCExtVideoShotTypeIdentifier@^{__CFString=}$kCGImagePropertyIPTCExtVideoShotTypeName@^{__CFString=}$kCGImagePropertyIPTCExtVideoStreamsCount@^{__CFString=}$kCGImagePropertyIPTCExtVisualColor@^{__CFString=}$kCGImagePropertyIPTCExtWorkflowTag@^{__CFString=}$kCGImagePropertyIPTCExtWorkflowTagCvId@^{__CFString=}$kCGImagePropertyIPTCExtWorkflowTagCvTermId@^{__CFString=}$kCGImagePropertyIPTCExtWorkflowTagCvTermName@^{__CFString=}$kCGImagePropertyIPTCExtWorkflowTagCvTermRefinedAbout@^{__CFString=}$kCGImagePropertyIPTCFixtureIdentifier@^{__CFString=}$kCGImagePropertyIPTCHeadline@^{__CFString=}$kCGImagePropertyIPTCImageOrientation@^{__CFString=}$kCGImagePropertyIPTCImageType@^{__CFString=}$kCGImagePropertyIPTCKeywords@^{__CFString=}$kCGImagePropertyIPTCLanguageIdentifier@^{__CFString=}$kCGImagePropertyIPTCObjectAttributeReference@^{__CFString=}$kCGImagePropertyIPTCObjectCycle@^{__CFString=}$kCGImagePropertyIPTCObjectName@^{__CFString=}$kCGImagePropertyIPTCObjectTypeReference@^{__CFString=}$kCGImagePropertyIPTCOriginalTransmissionReference@^{__CFString=}$kCGImagePropertyIPTCOriginatingProgram@^{__CFString=}$kCGImagePropertyIPTCProgramVersion@^{__CFString=}$kCGImagePropertyIPTCProvinceState@^{__CFString=}$kCGImagePropertyIPTCReferenceDate@^{__CFString=}$kCGImagePropertyIPTCReferenceNumber@^{__CFString=}$kCGImagePropertyIPTCReferenceService@^{__CFString=}$kCGImagePropertyIPTCReleaseDate@^{__CFString=}$kCGImagePropertyIPTCReleaseTime@^{__CFString=}$kCGImagePropertyIPTCRightsUsageTerms@^{__CFString=}$kCGImagePropertyIPTCScene@^{__CFString=}$kCGImagePropertyIPTCSource@^{__CFString=}$kCGImagePropertyIPTCSpecialInstructions@^{__CFString=}$kCGImagePropertyIPTCStarRating@^{__CFString=}$kCGImagePropertyIPTCSubLocation@^{__CFString=}$kCGImagePropertyIPTCSubjectReference@^{__CFString=}$kCGImagePropertyIPTCSupplementalCategory@^{__CFString=}$kCGImagePropertyIPTCTimeCreated@^{__CFString=}$kCGImagePropertyIPTCUrgency@^{__CFString=}$kCGImagePropertyIPTCWriterEditor@^{__CFString=}$kCGImagePropertyImageCount@^{__CFString=}$kCGImagePropertyImages@^{__CFString=}$kCGImagePropertyIsFloat@^{__CFString=}$kCGImagePropertyIsIndexed@^{__CFString=}$kCGImagePropertyIsSticker$kCGImagePropertyJFIFDensityUnit@^{__CFString=}$kCGImagePropertyJFIFDictionary@^{__CFString=}$kCGImagePropertyJFIFIsProgressive@^{__CFString=}$kCGImagePropertyJFIFVersion@^{__CFString=}$kCGImagePropertyJFIFXDensity@^{__CFString=}$kCGImagePropertyJFIFYDensity@^{__CFString=}$kCGImagePropertyMakerAppleDictionary@^{__CFString=}$kCGImagePropertyMakerCanonAspectRatioInfo@^{__CFString=}$kCGImagePropertyMakerCanonCameraSerialNumber@^{__CFString=}$kCGImagePropertyMakerCanonContinuousDrive@^{__CFString=}$kCGImagePropertyMakerCanonDictionary@^{__CFString=}$kCGImagePropertyMakerCanonFirmware@^{__CFString=}$kCGImagePropertyMakerCanonFlashExposureComp@^{__CFString=}$kCGImagePropertyMakerCanonImageSerialNumber@^{__CFString=}$kCGImagePropertyMakerCanonLensModel@^{__CFString=}$kCGImagePropertyMakerCanonOwnerName@^{__CFString=}$kCGImagePropertyMakerFujiDictionary@^{__CFString=}$kCGImagePropertyMakerMinoltaDictionary@^{__CFString=}$kCGImagePropertyMakerNikonCameraSerialNumber@^{__CFString=}$kCGImagePropertyMakerNikonColorMode@^{__CFString=}$kCGImagePropertyMakerNikonDictionary@^{__CFString=}$kCGImagePropertyMakerNikonDigitalZoom@^{__CFString=}$kCGImagePropertyMakerNikonFlashExposureComp@^{__CFString=}$kCGImagePropertyMakerNikonFlashSetting@^{__CFString=}$kCGImagePropertyMakerNikonFocusDistance@^{__CFString=}$kCGImagePropertyMakerNikonFocusMode@^{__CFString=}$kCGImagePropertyMakerNikonISOSelection@^{__CFString=}$kCGImagePropertyMakerNikonISOSetting@^{__CFString=}$kCGImagePropertyMakerNikonImageAdjustment@^{__CFString=}$kCGImagePropertyMakerNikonLensAdapter@^{__CFString=}$kCGImagePropertyMakerNikonLensInfo@^{__CFString=}$kCGImagePropertyMakerNikonLensType@^{__CFString=}$kCGImagePropertyMakerNikonQuality@^{__CFString=}$kCGImagePropertyMakerNikonSharpenMode@^{__CFString=}$kCGImagePropertyMakerNikonShootingMode@^{__CFString=}$kCGImagePropertyMakerNikonShutterCount@^{__CFString=}$kCGImagePropertyMakerNikonWhiteBalanceMode@^{__CFString=}$kCGImagePropertyMakerOlympusDictionary@^{__CFString=}$kCGImagePropertyMakerPentaxDictionary@^{__CFString=}$kCGImagePropertyNamedColorSpace@^{__CFString=}$kCGImagePropertyOpenEXRAspectRatio@^{__CFString=}$kCGImagePropertyOpenEXRDictionary@^{__CFString=}$kCGImagePropertyOrientation@^{__CFString=}$kCGImagePropertyPNGAuthor@^{__CFString=}$kCGImagePropertyPNGChromaticities@^{__CFString=}$kCGImagePropertyPNGComment@^{__CFString=}$kCGImagePropertyPNGCompressionFilter@^{__CFString=}$kCGImagePropertyPNGCopyright@^{__CFString=}$kCGImagePropertyPNGCreationTime@^{__CFString=}$kCGImagePropertyPNGDescription@^{__CFString=}$kCGImagePropertyPNGDictionary@^{__CFString=}$kCGImagePropertyPNGDisclaimer@^{__CFString=}$kCGImagePropertyPNGGamma@^{__CFString=}$kCGImagePropertyPNGInterlaceType@^{__CFString=}$kCGImagePropertyPNGModificationTime@^{__CFString=}$kCGImagePropertyPNGSoftware@^{__CFString=}$kCGImagePropertyPNGSource@^{__CFString=}$kCGImagePropertyPNGTitle@^{__CFString=}$kCGImagePropertyPNGWarning@^{__CFString=}$kCGImagePropertyPNGXPixelsPerMeter@^{__CFString=}$kCGImagePropertyPNGYPixelsPerMeter@^{__CFString=}$kCGImagePropertyPNGsRGBIntent@^{__CFString=}$kCGImagePropertyPixelFormat@^{__CFString=}$kCGImagePropertyPixelHeight@^{__CFString=}$kCGImagePropertyPixelWidth@^{__CFString=}$kCGImagePropertyPrimaryImage@^{__CFString=}$kCGImagePropertyProfileName@^{__CFString=}$kCGImagePropertyRawDictionary@^{__CFString=}$kCGImagePropertyTIFFArtist@^{__CFString=}$kCGImagePropertyTIFFCompression@^{__CFString=}$kCGImagePropertyTIFFCopyright@^{__CFString=}$kCGImagePropertyTIFFDateTime@^{__CFString=}$kCGImagePropertyTIFFDictionary@^{__CFString=}$kCGImagePropertyTIFFDocumentName@^{__CFString=}$kCGImagePropertyTIFFHostComputer@^{__CFString=}$kCGImagePropertyTIFFImageDescription@^{__CFString=}$kCGImagePropertyTIFFMake@^{__CFString=}$kCGImagePropertyTIFFModel@^{__CFString=}$kCGImagePropertyTIFFOrientation@^{__CFString=}$kCGImagePropertyTIFFPhotometricInterpretation@^{__CFString=}$kCGImagePropertyTIFFPrimaryChromaticities@^{__CFString=}$kCGImagePropertyTIFFResolutionUnit@^{__CFString=}$kCGImagePropertyTIFFSoftware@^{__CFString=}$kCGImagePropertyTIFFTileLength@^{__CFString=}$kCGImagePropertyTIFFTileWidth@^{__CFString=}$kCGImagePropertyTIFFTransferFunction@^{__CFString=}$kCGImagePropertyTIFFWhitePoint@^{__CFString=}$kCGImagePropertyTIFFXResolution@^{__CFString=}$kCGImagePropertyTIFFYResolution@^{__CFString=}$kCGImagePropertyThumbnailImages@^{__CFString=}$kCGImagePropertyWidth@^{__CFString=}$kCGImageSourceCreateThumbnailFromImageAlways@^{__CFString=}$kCGImageSourceCreateThumbnailFromImageIfAbsent@^{__CFString=}$kCGImageSourceCreateThumbnailWithTransform@^{__CFString=}$kCGImageSourceShouldAllowFloat@^{__CFString=}$kCGImageSourceShouldCache@^{__CFString=}$kCGImageSourceShouldCacheImmediately@^{__CFString=}$kCGImageSourceSubsampleFactor@^{__CFString=}$kCGImageSourceThumbnailMaxPixelSize@^{__CFString=}$kCGImageSourceTypeIdentifierHint@^{__CFString=}$"""
enums = """$IIO_HAS_IOSURFACE@1$IMAGEIO_PNG_ALL_FILTERS@248$IMAGEIO_PNG_FILTER_AVG@64$IMAGEIO_PNG_FILTER_NONE@8$IMAGEIO_PNG_FILTER_PAETH@128$IMAGEIO_PNG_FILTER_SUB@16$IMAGEIO_PNG_FILTER_UP@32$IMAGEIO_PNG_NO_FILTERS@0$kCGImageAnimationStatus_AllocationFailure@-22144$kCGImageAnimationStatus_CorruptInputImage@-22141$kCGImageAnimationStatus_IncompleteInputImage@-22143$kCGImageAnimationStatus_ParameterError@-22140$kCGImageAnimationStatus_UnsupportedFormat@-22142$kCGImageMetadataErrorBadArgument@2$kCGImageMetadataErrorConflictingArguments@3$kCGImageMetadataErrorPrefixConflict@4$kCGImageMetadataErrorUnknown@0$kCGImageMetadataErrorUnsupportedFormat@1$kCGImageMetadataTypeAlternateArray@4$kCGImageMetadataTypeAlternateText@5$kCGImageMetadataTypeArrayOrdered@3$kCGImageMetadataTypeArrayUnordered@2$kCGImageMetadataTypeDefault@0$kCGImageMetadataTypeInvalid@-1$kCGImageMetadataTypeString@1$kCGImageMetadataTypeStructure@6$kCGImagePropertyOrientationDown@3$kCGImagePropertyOrientationDownMirrored@4$kCGImagePropertyOrientationLeft@8$kCGImagePropertyOrientationLeftMirrored@5$kCGImagePropertyOrientationRight@6$kCGImagePropertyOrientationRightMirrored@7$kCGImagePropertyOrientationUp@1$kCGImagePropertyOrientationUpMirrored@2$kCGImageStatusComplete@0$kCGImageStatusIncomplete@-1$kCGImageStatusInvalidData@-4$kCGImageStatusReadingHeader@-2$kCGImageStatusUnexpectedEOF@-5$kCGImageStatusUnknownType@-3$"""
misc.update({})
functions = {
    "CGImageMetadataSetValueMatchingImageProperty": (
        b"B^{CGImageMetadata=}^{__CFString=}^{__CFString=}@",
    ),
    "CGImageMetadataCreateFromXMPData": (
        b"^{CGImageMetadata=}^{__CFData=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageSourceGetCount": (sel32or64(b"L^{CGImageSource=}", b"Q^{CGImageSource=}"),),
    "CGImageSourceCreateWithData": (
        b"^{CGImageSource=}^{__CFData=}^{__CFDictionary=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageSourceCopyTypeIdentifiers": (
        b"^{__CFArray=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageMetadataCopyTagWithPath": (
        b"^{CGImageMetadataTag=}^{CGImageMetadata=}^{CGImageMetadataTag=}^{__CFString=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageMetadataGetTypeID": (sel32or64(b"L", b"Q"),),
    "CGImageDestinationAddAuxiliaryDataInfo": (
        b"v^{CGImageDestination=}^{__CFString=}^{__CFDictionary=}",
    ),
    "CGImageMetadataTagGetType": (b"i^{CGImageMetadataTag=}",),
    "CGImageSourceGetStatus": (b"i^{CGImageSource=}",),
    "CGImageMetadataCreateXMPData": (
        b"^{__CFData=}^{CGImageMetadata=}^{__CFDictionary=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageSourceCreateImageAtIndex": (
        sel32or64(
            b"^{CGImage=}^{CGImageSource=}L^{__CFDictionary=}",
            b"^{CGImage=}^{CGImageSource=}Q^{__CFDictionary=}",
        ),
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageMetadataTagCopyNamespace": (
        b"^{__CFString=}^{CGImageMetadataTag=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageSourceCopyMetadataAtIndex": (
        sel32or64(
            b"^{CGImageMetadata=}^{CGImageSource=}L^{__CFDictionary=}",
            b"^{CGImageMetadata=}^{CGImageSource=}Q^{__CFDictionary=}",
        ),
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageDestinationCreateWithURL": (
        sel32or64(
            b"^{CGImageDestination=}^{__CFURL=}^{__CFString=}L^{__CFDictionary=}",
            b"^{CGImageDestination=}^{__CFURL=}^{__CFString=}Q^{__CFDictionary=}",
        ),
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageSourceRemoveCacheAtIndex": (
        sel32or64(b"v^{CGImageSource=}L", b"v^{CGImageSource=}Q"),
    ),
    "CGImageSourceCreateThumbnailAtIndex": (
        sel32or64(
            b"^{CGImage=}^{CGImageSource=}L^{__CFDictionary=}",
            b"^{CGImage=}^{CGImageSource=}Q^{__CFDictionary=}",
        ),
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageSourceCreateIncremental": (
        b"^{CGImageSource=}^{__CFDictionary=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageSourceCopyPropertiesAtIndex": (
        sel32or64(
            b"^{__CFDictionary=}^{CGImageSource=}L^{__CFDictionary=}",
            b"^{__CFDictionary=}^{CGImageSource=}Q^{__CFDictionary=}",
        ),
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageSourceUpdateData": (b"v^{CGImageSource=}^{__CFData=}B",),
    "CGImageDestinationSetProperties": (b"v^{CGImageDestination=}^{__CFDictionary=}",),
    "CGImageMetadataSetTagWithPath": (
        b"B^{CGImageMetadata=}^{CGImageMetadataTag=}^{__CFString=}^{CGImageMetadataTag=}",
    ),
    "CGAnimateImageAtURLWithBlock": (
        b"i^{__CFURL=}^{__CFDictionary=}@?",
        "",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": "^v"},
                            1: {"type": "l"},
                            2: {"type": "@"},
                            3: {"type": "o^B"},
                        },
                    }
                }
            }
        },
    ),
    "CGImageDestinationCreateWithDataConsumer": (
        sel32or64(
            b"^{CGImageDestination=}^{CGDataConsumer=}^{__CFString=}L^{__CFDictionary=}",
            b"^{CGImageDestination=}^{CGDataConsumer=}^{__CFString=}Q^{__CFDictionary=}",
        ),
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageDestinationCopyTypeIdentifiers": (
        b"^{__CFArray=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageDestinationCreateWithData": (
        sel32or64(
            b"^{CGImageDestination=}^{__CFData=}^{__CFString=}L^{__CFDictionary=}",
            b"^{CGImageDestination=}^{__CFData=}^{__CFString=}Q^{__CFDictionary=}",
        ),
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageSourceCopyAuxiliaryDataInfoAtIndex": (
        b"^{__CFDictionary=}^{CGImageSource=}Q^{__CFString=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGAnimateImageDataWithBlock": (
        b"i^{__CFData=}^{__CFDictionary=}@?",
        "",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": "^v"},
                            1: {"type": "l"},
                            2: {"type": "@"},
                            3: {"type": "o^B"},
                        },
                    }
                }
            }
        },
    ),
    "CGImageSourceGetTypeID": (sel32or64(b"L", b"Q"),),
    "CGImageDestinationAddImageAndMetadata": (
        b"v^{CGImageDestination=}^{CGImage=}^{CGImageMetadata=}^{__CFDictionary=}",
    ),
    "CGImageMetadataTagCreate": (
        b"^{CGImageMetadataTag=}^{__CFString=}^{__CFString=}^{__CFString=}i@",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageDestinationCopyImageSource": (
        b"B^{CGImageDestination=}^{CGImageSource=}^{__CFDictionary=}^^{__CFError=}",
        "",
        {
            "retval": {"already_cfretained": True},
            "arguments": {
                3: {
                    "already_cfretained": True,
                    "type_modifier": "o",
                    "null_accepted": True,
                }
            },
        },
    ),
    "CGImageSourceUpdateDataProvider": (b"v^{CGImageSource=}^{CGDataProvider=}B",),
    "CGImageDestinationAddImage": (
        b"v^{CGImageDestination=}^{CGImage=}^{__CFDictionary=}",
    ),
    "CGImageMetadataRemoveTagWithPath": (
        b"B^{CGImageMetadata=}^{CGImageMetadataTag=}^{__CFString=}",
    ),
    "CGImageSourceGetStatusAtIndex": (
        sel32or64(b"i^{CGImageSource=}L", b"i^{CGImageSource=}Q"),
    ),
    "CGImageSourceCreateWithURL": (
        b"^{CGImageSource=}^{__CFURL=}^{__CFDictionary=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageMetadataTagCopyValue": (
        b"@^{CGImageMetadataTag=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageMetadataCopyTags": (
        b"^{__CFArray=}^{CGImageMetadata=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageMetadataCopyStringValueWithPath": (
        b"^{__CFString=}^{CGImageMetadata=}^{CGImageMetadataTag=}^{__CFString=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageSourceGetType": (b"^{__CFString=}^{CGImageSource=}",),
    "CGImageDestinationAddImageFromSource": (
        sel32or64(
            b"v^{CGImageDestination=}^{CGImageSource=}L^{__CFDictionary=}",
            b"v^{CGImageDestination=}^{CGImageSource=}Q^{__CFDictionary=}",
        ),
    ),
    "CGImageDestinationGetTypeID": (sel32or64(b"L", b"Q"),),
    "CGImageDestinationFinalize": (b"B^{CGImageDestination=}",),
    "CGImageSourceGetPrimaryImageIndex": (b"Q^{CGImageSource=}",),
    "CGImageSourceCopyProperties": (
        b"^{__CFDictionary=}^{CGImageSource=}^{__CFDictionary=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageMetadataTagGetTypeID": (sel32or64(b"L", b"Q"),),
    "CGImageMetadataSetValueWithPath": (
        b"B^{CGImageMetadata=}^{CGImageMetadataTag=}^{__CFString=}@",
    ),
    "CGImageMetadataCopyTagMatchingImageProperty": (
        b"^{CGImageMetadataTag=}^{CGImageMetadata=}^{__CFString=}^{__CFString=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageMetadataRegisterNamespaceForPrefix": (
        b"B^{CGImageMetadata=}^{__CFString=}^{__CFString=}^^{__CFError=}",
        "",
        {
            "arguments": {
                3: {
                    "already_cfretained": True,
                    "type_modifier": "o",
                    "null_accepted": True,
                }
            }
        },
    ),
    "CGImageMetadataTagCopyPrefix": (
        b"^{__CFString=}^{CGImageMetadataTag=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageMetadataTagCopyName": (
        b"^{__CFString=}^{CGImageMetadataTag=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageMetadataEnumerateTagsUsingBlock": (
        b"v^{CGImageMetadata=}^{__CFString=}^{__CFDictionary=}@?",
        "",
        {
            "arguments": {
                3: {
                    "callable": {
                        "retval": {"type": b"B"},
                        "arguments": {
                            0: {"type": "^v"},
                            1: {"type": "@"},
                            2: {"type": "@"},
                        },
                    }
                }
            }
        },
    ),
    "CGImageMetadataCreateMutable": (
        b"^{CGImageMetadata=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageSourceCreateWithDataProvider": (
        b"^{CGImageSource=}^{CGDataProvider=}^{__CFDictionary=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageMetadataTagCopyQualifiers": (
        b"^{__CFArray=}^{CGImageMetadataTag=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageMetadataCreateMutableCopy": (
        b"^{CGImageMetadata=}^{CGImageMetadata=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
}
aliases = {
    "IMAGEIO_AVAILABLE_STARTING": "__OSX_AVAILABLE_STARTING",
    "IMAGEIO_AVAILABLE_BUT_DEPRECATED": "__OSX_AVAILABLE_BUT_DEPRECATED",
    "_iio_Nullable": "_Nullable",
    "_iio_Nonnull": "_Nonnull",
}
cftypes = [
    ("CGImageSourceRef", b"^{CGImageSource=}", "CGImageSourceGetTypeID", None),
    (
        "CGImageDestinationRef",
        b"^{CGImageDestination=}",
        "CGImageDestinationGetTypeID",
        None,
    ),
    (
        "CGImageMetadataTagRef",
        b"^{CGImageMetadataTag=}",
        "CGImageMetadataTagGetTypeID",
        None,
    ),
    ("CGImageMetadataRef", b"^{CGImageMetadata=}", "CGImageMetadataGetTypeID", None),
]
expressions = {}

# END OF FILE
