import AppDelegate  # noqa: F401
import Circle  # noqa: F401
import Extras  # noqa: F401
import ShadowOffsetView  # noqa: F401
import TLayerDemo  # noqa: F401
import TLayerView  # noqa: F401
from PyObjCTools import AppHelper


AppHelper.runEventLoop()
